@@ -29,7 +29,7 @@ print OUT <<'!NO!SUBS!';
 # Info-URL: http://degraaff.org/checkbot/
 # Comments to: checkbot@degraaff.org
 #
-# $Id: checkbot.pl,v 1.61 2000/06/29 19:56:48 graaff Exp $
+# $Id: checkbot.pl,v 1.62 2000/09/24 13:51:46 graaff Exp $
 # (Log information can be found at the end of the script)
 
 require 5.004;
@@ -46,10 +46,12 @@ Checkbot - WWW Link Verifier
 checkbot [B<--debug>] [B<--help>] [B<--verbose>] [B<--url> start URL] 
          [B<--match> match string] [B<--exclude> exclude string]
          [B<--proxy> proxy URL] [B<--internal-only>]
-         [B<--ignore> ignore string] [B<-file> file name] 
+         [B<--ignore> ignore string] [B<-file> file name]
+         [B<--style> style file URL]
          [B<--mailto> email address]
          [B<--note> note] [B<--sleep> seconds] [B<--timeout> timeout]
          [B<--interval> seconds] [B<--dontwarn> HTTP responde codes]
+         [B<--enable-virtual>]
          [start URLs]
 
 =head1 DESCRIPTION
@@ -155,12 +157,29 @@ gradually extend it towards the maximum interval.
 
 Write the summary pages into file I<file name>. Default is C<checkbot.html>.
 
+=item --style <URL of style file>
+
+When this option is used, Checkbot embeds this URL as a link to a
+style file on each page it writes. This makes it easy to customize the
+layout of pages generated by Checkbot.
+
 =item --dontwarn <HTTP response codes regular expression>
 
 Don't include warnings on the result pages for those HTTP response
 codes which match the regular expression. For instance, --dontwarn
 "(301|404)" would not include 301 and 404 response codes.
 
+=item --enable-virtual
+
+This option enables dealing with virtual servers. Checkbot then
+assumes that all hostnames for internal servers are unique, even
+though their IP addresses may be the same. Normally Checkbot uses the
+IP address to distinguish servers. This has the advantage that if a
+server has two names (e.g. www and bamboozle) its pages only get
+checked once. When you want to check multiple virtual servers this
+causes problems, which this feature work around by using the hostname
+to distinguish the server.
+
 =back
 
 =head1 PREREQUISITES
@@ -196,7 +215,7 @@ my %problems = ();
 
 # Version information
 my $VERSION;
-( $VERSION ) = sprintf("%d.%02d", q$Revision: 1.61 $ =~ /(\d+)\.(\d+)/);
+( $VERSION ) = sprintf("%d.%02d", q$Revision: 1.62 $ =~ /(\d+)\.(\d+)/);
 
 # If on a Mac we should ask for the arguments through some MacPerl stuff
 if ($^O eq 'MacOS') {
@@ -236,7 +255,7 @@ sub check_options {
 
   # Get command-line arguments
   use Getopt::Long;
-  my $result = GetOptions(qw(debug help verbose url=s match=s exclude|x=s file=s ignore|z=s mailto|M=s note|N=s proxy=s internal-only sleep=i timeout=i interval=i dontwarn=s));
+  my $result = GetOptions(qw(debug help verbose url=s match=s exclude|x=s file=s style=s ignore|z=s mailto|M=s note|N=s proxy=s internal-only sleep=i timeout=i interval=i dontwarn=s enable-virtual));
 
   # Handle arguments, some are mandatory, some have defaults
   &print_help if (($main::opt_help && $main::opt_help) 
@@ -246,6 +265,7 @@ sub check_options {
   $main::opt_sleep = 2 unless defined($main::opt_sleep) && length($main::opt_sleep);
   $main::opt_interval = 10800 unless defined $main::opt_interval and length $main::opt_interval;
   $main::opt_dontwarn = "xxx" unless defined $main::opt_dontwarn and length $main::opt_dontwarn;
+  $main::opt_enable_virtual = 0 unless defined $main::opt_enable_virtual;
   # The default for opt_match will be set later, because we might want
   # to muck with opt_url first.
 
@@ -613,7 +633,7 @@ sub check_external {
 
           # Netscape Enterprise has been seen returning 500 and even 404
           # (yes, 404!!) in response to HEAD requests
-          elsif ($response->server =~ /^Netscape-Enterprise/o &&
+          elsif (defined $response->server and $response->server =~ /^Netscape-Enterprise/o &&
            $response->code =~ /^(404|500)$/o) {
             print STDERR "Unreliable response to HEAD request; retrying\n"
              if $main::opt_verbose;
@@ -621,6 +641,24 @@ sub check_external {
             next;
           }
 
+          # JavaWebServer/1.1.3 has been seen returning 500
+          elsif (defined $response->server and $response->server =~ /^JavaWebServer/o &&
+           $response->code =~ /^500$/o) {
+            print STDERR "Unreliable response to HEAD request; retrying\n"
+             if $main::opt_verbose;
+            $main::servers_get_only{$url->netloc}++;
+            next;
+          }
+
+          # ALEPH has been seen returning 500
+          elsif (defined $response->server and $response->server =~ /^ALEPH/o &&
+           $response->code =~ /^500$/o) {
+            print STDERR "Unreliable response to HEAD request; retrying\n"
+             if $main::opt_verbose;
+            $main::servers_get_only{$url->netloc}++;
+            next;
+          }
+
           # If a HEAD request resulted in nothing noteworthy, no need for
           # any further attempts
           else { last; }
@@ -667,7 +705,9 @@ sub create_page {
       int($main::cp_int * 60 / 2 - 5);
     }
 
-    print OUT "<title>Checkbot report</title></head>\n";
+    print OUT "<title>Checkbot report</title>\n";
+    print OUT "<link rel=\"StyleSheet\" type=\"text/css\" href=\"$main::opt_style\">\n" if defined $main::opt_style;
+    print OUT "</head>\n";
     print OUT "<h1><em>Checkbot</em>: main report</h1>\n";
 
     # Show the status of this checkbot session
@@ -720,6 +760,9 @@ sub create_page {
     print OUT "<tr><th align=left>--match</th><td>Match regular expression</td><td>$main::opt_match</td></tr>\n";
     print OUT "<tr><th align=left>--exclude</th><td>Exclude regular expression</td><td>$main::opt_exclude</td></tr>\n" if defined $main::opt_exclude;
     print OUT "<tr><th align=left>--ignore</th><td>Ignore regular expression</td><td>$main::opt_ignore</td></tr>\n" if defined $main::opt_ignore;
+    print OUT "<tr><th align=left>--dontwarn</th><td>Don't warn for these codes</td><td>$main::opt_dontwarn</td></tr>\n" if $main::opt_dontwarn ne 'xxx';
+    print OUT "<tr><th align=left>--enable-virtual</th><td>Use virtual names only</td><td>yes</td></tr>\n" if $main::opt_enable_virtual;
+    print OUT "<tr><th align=left>--internal-only</th><td>Check only internal links</td><td>yes</td></tr>\n" if defined $main::opt_internal_only;
     
     print OUT "</table>\n";
 
@@ -783,16 +826,19 @@ sub add_checked {
   my $item;
   my $result = 0;
 
-  # Substitute hostname with IP-address. This keeps us from checking 
-  # the same pages for each name of the server, wasting time & resources.
-  my $url = URI->new($urlstr);
+  unless ($main::opt_enable_virtual) {
+    # Substitute hostname with IP-address. This keeps us from checking
+    # the same pages for each name of the server, wasting time & resources.
+    # Only do this if we are not dealing with virtual servers.
+    my $url = URI->new($urlstr);
 
-  # TODO: This should be fixed in a more sane way. Really the URI
-  # class should deal with stuff like this. Checkbot can't be expected
-  # to know which URI schemes do have a host component and which types
-  # don't.
-  $url->host(ip_address($url->host)) if $url->scheme =~ /^(http|ftp|gopher|https|ldap|news|nntp|pop|rlogin|snews|telnet)$/;
-  $urlstr = $url->as_string;
+    # TODO: This should be fixed in a more sane way. Really the URI
+    # class should deal with stuff like this. Checkbot can't be expected
+    # to know which URI schemes do have a host component and which types
+    # don't.
+    $url->host(ip_address($url->host)) if $url->scheme =~ /^(http|ftp|gopher|https|ldap|news|nntp|pop|rlogin|snews|telnet)$/;
+    $urlstr = $url->as_string;
+  }
 
   if (defined $main::checked{$urlstr}) {
     $result = 1;
@@ -917,6 +963,7 @@ sub print_server {
     printf SERVER "<META HTTP-EQUIV=\"Refresh\" CONTENT = %d>\n",
       int($main::cp_int * 60 / 2 - 5);
   }
+  print SERVER "<link rel=\"StyleSheet\" type=\"text/css\" href=\"$main::opt_style\">\n" if defined $main::opt_style;
   print SERVER "<title>Checkbot: output for server $server</title></head>\n";
   print SERVER "<body><h2><em>Checkbot</em>: report for server <tt>$server</tt></h2>\n";
   print SERVER "Go To: <a href=\"$main::file\">Main report page</a>.\n";
@@ -1024,9 +1071,11 @@ sub check_point {
 
 sub send_mail {
   my $msg = new Mail::Send;
+  my $sub = 'Checkbot results are available';
+  $sub .= " for $main::opt_url" if defined $main::opt_url;
 
   $msg->to($main::opt_mailto);
-  $msg->subject("Checkbot results for $main::opt_url available.");
+  $msg->subject($sub);
 
   my $fh = $msg->open;
 
@@ -1071,6 +1120,7 @@ sub print_help {
   print "  --timeout seconds  Timeout for http requests in seconds (default 120)\n";
   print "  --interval seconds Maximum time interval between updates (default 10800)\n";
   print "  --dontwarn codes   Do not write warnings for these HTTP response codes\n";
+  print "  --enable-virtual   Use only virtual names, not IP numbers for servers\n";
   print "\n";
   print "Options --match, --exclude, and --ignore can take a perl regular expression\nas their argument\n\n";
   print "Use 'perldoc checkbot' for more verbose documentation.\n\n";
@@ -1115,6 +1165,9 @@ sub count_problems {
 
 
 # $Log: checkbot.pl,v $
+# Revision 1.62  2000/09/24 13:51:46  graaff
+# Checkbot 1.62.
+#
 # Revision 1.61  2000/06/29 19:56:48  graaff
 # Updated Makefile.PL
 # Use GET instead of HEAD for confused servers.
